﻿// $Id: FormMain.cs 21 2008-03-27 02:38:24Z arcao $
//
// Copyright (C) 2008 Martin "Arcao" Sloup <arcao@arcao.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections;
using System.Drawing;
using System.IO;
using System.Windows.Forms;
using BrightIdeasSoftware;
using OSD.PluginBase;

namespace OSD.Program
{
	internal partial class FormMain : Form
	{
		private DownloadItems downloadItems;
		private Plugins plugins;
		private Settings settings;

		//private bool isInitState = true;

		public FormMain()
		{
			InitializeComponent();
			Init();
		}

		private void FormMain_Load(object sender, EventArgs e) {}

		private void stopStartToolStripButton_Click(object sender, EventArgs e)
		{
			UpdateDownload();
			settings.setBool("osd/main/is_download_stopped", stopStartToolStripButton.Checked);
		}

		private void fileToolStripMenuItem_Click(object sender, EventArgs e) {}

		private void Init()
		{
			//uprava fontu
			Util.SetSystemFont(Controls);

			//zjistime cestu k profilu s nastavenim
			string appSettingsPath = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), "OSD");
			//vytvorime adresar
			Directory.CreateDirectory(appSettingsPath);

			//nastavime cestu k souboru s nastavenim
			Settings.default_filename = Path.Combine(appSettingsPath, "config.xml");

			settings = Settings.getInstance();

			downloadItems = new DownloadItems(settings);
			downloadItems.DownloadComplete += OnDownloadComplete;
			downloadItems.DownloadStatusChanged += OnDownloadStatusChanged;
			downloadItems.PositionChanged += OnPositionChanged;
			downloadItems.StatusChanged += OnStatusChanged;
			downloadItems.LoadList();

			plugins = Plugins.GetInstance();
			plugins.Load();

			LoadSettings();

			#region list Column delegates and other list settings

			list.CustomSorter = delegate { };

			urlOlvColumn.AspectGetter = delegate(object rowObject) { return ((DownloadItem) rowObject).Url; };

			urlOlvColumn.ImageGetter = delegate(object rowObject)
			{
				switch(((DownloadItem) rowObject).Status)
				{
					case DownloadItemStatus.Queued:
						return 0;
					case DownloadItemStatus.Downloading:
						return 1;
					case DownloadItemStatus.Deleted:
						return 2;
					case DownloadItemStatus.Waiting:
						return 3;
					case DownloadItemStatus.Downloaded:
						return 4;
					default:
						return -1;
				}
			};

			statusOlvColumn.AspectGetter = delegate(object rowObject)
			{
				switch(((DownloadItem) rowObject).Status)
				{
					case DownloadItemStatus.Queued:
						return "Čekám ve frontě...";
					case DownloadItemStatus.Downloading:
						return ((DownloadItem) rowObject).StatusText.Length > 0 ? ((DownloadItem) rowObject).StatusText : "Stahuji...";
					case DownloadItemStatus.Deleted:
						return "Smazáno...";
					case DownloadItemStatus.Waiting:
						return
							((DownloadItem) rowObject).StatusText.Length > 0
								? ((DownloadItem) rowObject).StatusText
								: "Čekám na volný slot...";
					case DownloadItemStatus.Downloaded:
						return "Stáhnuto...";
					default:
						return "";
				}
			};

			progressOlvColumn.AspectGetter = delegate(object rowObject)
			{
				DownloadItem di = (DownloadItem) rowObject;
				if (di.Download == null || di.Status == DownloadItemStatus.Downloaded ||
				    di.Download.Status != DownloadStatus.Downloading)
					return -1;
				else
					return di.Download.Percents;
			};
			downloadedBytesOlvColumn.AspectGetter = delegate(object rowObject)
			{
				DownloadItem di = (DownloadItem) rowObject;
				if (di.Download == null || di.Status == DownloadItemStatus.Downloaded ||
				    di.Download.Status != DownloadStatus.Downloading)
					return "0 B";
				else
					return Util.SizeFormat(di.Download.Position);
			};
			fileSizeOlvColumn.AspectGetter = delegate(object rowObject)
			{
				DownloadItem di = (DownloadItem) rowObject;
				if (di.Download == null || di.Status == DownloadItemStatus.Downloaded ||
				    di.Download.Status != DownloadStatus.Downloading)
					return "Neznámá";
				else
					return Util.SizeFormat(di.Download.Size);
			};
			BarRenderer br =
				new PercentBarRenderer(0, 100, new Pen(Color.Empty, 0), SystemColors.ActiveCaption,
				                       SystemColors.GradientActiveCaption);
			br.MaximumHeight = 100;
			br.MaximumWidth = 1500;
			br.BackgroundBrush = Brushes.Transparent;

			progressOlvColumn.Renderer = br;

			list.SetObjects(downloadItems);
			list.OwnerDraw = true;

			#endregion

			UpdateDownload();
			UpdateLayoutEnabling();

			//isInitState = false;
		}

		private void LoadSettings()
		{
			stopStartToolStripButton.Checked = settings.getBool("osd/main/is_download_stopped", false);

			Left = settings.getInt("osd/main/left", int.MinValue);
			Top = settings.getInt("osd/main/top", int.MinValue);

			// pokud je nastavena Top a Left, tak nastavime StartPosition na manual, cimz docilime,
			// ze se pri zobrazeni okna vezmou nase hodnoty Top a Left
			if (Top != int.MinValue && Left != int.MinValue)
				StartPosition = FormStartPosition.Manual;

			Width = settings.getInt("osd/main/width", Width);
			Height = settings.getInt("osd/main/height", Height);

			WindowState =
				(FormWindowState)
				Enum.Parse(typeof(FormWindowState), settings.getString("osd/main/window_state", WindowState.ToString()), true);

			// nastaveni sirky sloupcu u listu
			for (int i = 0; i < list.Columns.Count; i++)
				list.Columns[i].Width = settings.getInt(string.Format("osd/main/columns/column{0}", i), list.Columns[i].Width);
		}

		private void insertToolStripButton_Click(object sender, EventArgs e)
		{
			string url = addUrltoolStripSpringTextBox.Text;
			if (plugins.isPluginFor(url))
			{
				downloadItems.Add(url);
				UpdateList();
				UpdateDownload();
			}
			addUrltoolStripSpringTextBox.Text = string.Empty;
		}

		private void moveUpToolStripButton_Click(object sender, EventArgs e)
		{
			if (list.SelectedObjects.Count == 0 || downloadItems.IndexOf((DownloadItem) list.SelectedObjects[0]) == 0)
				return;

			for (int i = 0; i < list.SelectedObjects.Count; i++)
			{
				DownloadItem di = (DownloadItem) list.SelectedObjects[i];
				downloadItems.MoveUp(di);
			}
			UpdateList();
		}

		private void moveDownToolStripButton_Click(object sender, EventArgs e)
		{
			if (list.SelectedObjects.Count == 0 ||
			    downloadItems.IndexOf((DownloadItem) list.SelectedObjects[list.SelectedObjects.Count - 1]) ==
			    downloadItems.Count - 1)
				return;

			for (int i = list.SelectedObjects.Count - 1; i >= 0; i--)
			{
				DownloadItem di = (DownloadItem) list.SelectedObjects[i];
				downloadItems.MoveDown(di);
			}
			UpdateList();
		}

		private void deleteToolStripButton_Click(object sender, EventArgs e)
		{
			if (list.SelectedObjects.Count > 0)
			{
				foreach(DownloadItem di in list.SelectedObjects)
					downloadItems.Remove(di);
				UpdateList();
			}
		}

		private void fileSettingsToolStripMenuItem_Click(object sender, EventArgs e)
		{
			FormSettings f = new FormSettings();
			if (f.ShowDialog(this) == DialogResult.OK)
			{
				// znovu spustime stahovani, pokud bezi (pouzijeme nastaveni pluginu)
				foreach(DownloadItem di in downloadItems)
				{
					if (di.Status == DownloadItemStatus.Downloading)
						di.doDownload();
				}
			}
		}

		private void fileCloseToolStripMenuItem_Click(object sender, EventArgs e)
		{
			Close();
		}

		private void fileEditToolStripMenuItem_Click(object sender, EventArgs e)
		{
			editToolStripButton_Click(sender, e);
		}

		private void editToolStripButton_Click(object sender, EventArgs e)
		{
			if (list.SelectedObject == null) return;

			DownloadItem di = (DownloadItem) list.SelectedObject;

			FormEditItem f = new FormEditItem(di);
			f.ShowDialog(this);
			UpdateDownload();
		}

		private void fileRemoveToolStripMenuItem_Click(object sender, EventArgs e)
		{
			deleteToolStripButton_Click(sender, e);
		}

		private void addUrltoolStripSpringTextBox_TextChanged(object sender, EventArgs e)
		{
			insertToolStripButton.Enabled = (addUrltoolStripSpringTextBox.Text.Length > 0);
		}

		private void addUrltoolStripSpringTextBox_KeyPress(object sender, KeyPressEventArgs e)
		{
			if (e.KeyChar == (char) Keys.Return)
			{
				insertToolStripButton_Click(sender, e);
				e.Handled = true;
			}
		}

		private void FormMain_FormClosing(object sender, FormClosingEventArgs e)
		{
			SaveSettings();
		}

		private void SaveSettings()
		{
			downloadItems.SaveList();
			downloadItems.Dispose();

			//ulozime rozmery
			if (WindowState == FormWindowState.Normal)
			{
				settings.setInt("osd/main/top", Top);
				settings.setInt("osd/main/left", Left);
				settings.setInt("osd/main/width", Width);
				settings.setInt("osd/main/height", Height);
			} else
			{
				settings.setInt("osd/main/top", RestoreBounds.Top);
				settings.setInt("osd/main/left", RestoreBounds.Left);
				settings.setInt("osd/main/width", RestoreBounds.Width);
				settings.setInt("osd/main/height", RestoreBounds.Height);
			}

			settings.setString("osd/main/window_state", WindowState.ToString());

			// ulozime sirku sloupcu
			for (int i = 0; i < list.Columns.Count; i++)
				settings.setInt(string.Format("osd/main/columns/column{0}", i), list.Columns[i].Width);

			list.Clear();
			settings.Save();
		}

		private void UpdateList()
		{
			if (InvokeRequired)
			{
				Invoke(new UpdateCallback(UpdateList));
				return;
			}

			// prekreslime obsah, pouzijeme beginupdate a endupdate, cimz zamezime problemum s oznacovanim polozek behem prekreslovani
			list.BeginUpdate();
			ArrayList so = list.SelectedObjects;
			list.SetObjects(downloadItems);
			list.SelectedObjects = so;
			list.EndUpdate();
			UpdateLayoutEnabling();
		}

		private void UpdateDownload()
		{
			if (InvokeRequired)
			{
				Invoke(new UpdateCallback(UpdateDownload));
				return;
			}

			if (stopStartToolStripButton.Checked)
			{
				// pokud je nastaveno nestahovani, zmenime vsechny se stavem download nebo waiting na queued
				foreach(DownloadItem item in downloadItems)
				{
					if (item.Status == DownloadItemStatus.Downloading || item.Status == DownloadItemStatus.Waiting)
					{
						item.DestroyDownload();
						item.ResetStatus();
						list.RefreshObject(item);
					}
				}
			} else
			{
				// pokud neni nastaveno nestahovani, spustime stahovani na stanoveny pocet polozek

				int availableSlots = settings.getInt("osd/settings/download_threads", 1);

				//spocitame, kolikrat bezi jiz stahovani
				foreach(DownloadItem item in downloadItems)
				{
					if (item.Status == DownloadItemStatus.Downloading || item.Status == DownloadItemStatus.Waiting)
						availableSlots--;
				}

				//nyni dle poctu volych slotu spustime stahovani na polozkach se stavem queued
				if (availableSlots > 0)
				{
					foreach(DownloadItem item in downloadItems)
					{
						if (availableSlots <= 0) break;
						if (item.Status == DownloadItemStatus.Queued)
						{
							item.doDownload();
							availableSlots--;
						}
					}
				}
			}
		}

		private void UpdateLayoutEnabling()
		{
			if (InvokeRequired)
			{
				Invoke(new UpdateCallback(UpdateLayoutEnabling));
				return;
			}

			// je mozne upravovat
			editToolStripButton.Enabled = (list.SelectedObject != null);
			fileEditToolStripMenuItem.Enabled = editToolStripButton.Enabled;

			// je mozne mazat
			deleteToolStripButton.Enabled = (list.SelectedObjects.Count > 0);
			fileRemoveToolStripMenuItem.Enabled = deleteToolStripButton.Enabled;

			// je mozne posouvat nahoru
			moveUpToolStripButton.Enabled = (list.SelectedObjects.Count > 0 &&
			                                 downloadItems.IndexOf((DownloadItem) list.SelectedObjects[0]) != 0);
			// je mozne posouvat dolu
			moveDownToolStripButton.Enabled = (list.SelectedObjects.Count > 0 &&
			                                   downloadItems.IndexOf(
			                                   	(DownloadItem) list.SelectedObjects[list.SelectedObjects.Count - 1]) !=
			                                   downloadItems.Count - 1);
		}

		private void timeoutTimer_Tick(object sender, EventArgs e)
		{
			bool updateDownload = false;
			foreach(DownloadItem item in downloadItems)
			{
				if (item.TimeToGo > 0)
				{
					item.TimeToGo--;
					if (item.TimeToGo == 0 && item.Status == DownloadItemStatus.Waiting)
					{
						item.ResetStatus();
						updateDownload = true;
					}
					list.RefreshObject(item);
				}
			}
			if (updateDownload) UpdateDownload();
		}

		private void list_SelectionChanged(object sender, EventArgs e)
		{
			UpdateLayoutEnabling();
		}

		private void list_DoubleClick(object sender, EventArgs e)
		{
			editToolStripButton_Click(sender, e);
		}

		private void addUrltoolStripSpringTextBox_Enter(object sender, EventArgs e)
		{
			addUrltoolStripSpringTextBox.BackColor = Color.FromArgb(0xFF, 0xFF, 0xCC);
		}

		private void addUrltoolStripSpringTextBox_Leave(object sender, EventArgs e)
		{
			addUrltoolStripSpringTextBox.BackColor = SystemColors.Window;
		}

		private void addToolStripSplitButton_ButtonClick(object sender, EventArgs e)
		{
			addUrltoolStripSpringTextBox.Focus();
		}

		private void addToolBarByTyppingToolStripMenuItem_Click(object sender, EventArgs e)
		{
			addToolStripSplitButton_ButtonClick(sender, e);
		}

		private void fileAddByTyppingToolStripMenuItem_Click(object sender, EventArgs e)
		{
			addToolStripSplitButton_ButtonClick(sender, e);
		}

		private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
		{
			FormAbout f = new FormAbout();
			f.ShowDialog(this);
		}

		#region Events implementation

		private void OnStatusChanged(object sender, EventArgs e)
		{
			list.RefreshObject(sender);
			DownloadItemStatusChangedEventArgs dischea = (DownloadItemStatusChangedEventArgs) e;
			if (dischea.Status == DownloadItemStatus.Waiting)
				((DownloadItem) sender).Wait(settings.getInt("osd/settings/timeout_waiting", 300), true);
		}

		private void OnPositionChanged(object sender, EventArgs e)
		{
			list.RefreshObject(sender);
		}

		private void OnDownloadStatusChanged(object sender, EventArgs e)
		{
			if (((DownloadStatusChangedEventArgs) e).Status == DownloadStatus.Waiting)
			{
				WaitingStatusChangeEventArgs wschea = (WaitingStatusChangeEventArgs) e;
				((DownloadItem) sender).Wait(
					(wschea.SuggestedWaiting > 0) ? wschea.SuggestedWaiting : settings.getInt("osd/settings/timeout_waiting", 300),
					false);
			}
			list.RefreshObject(sender);
		}

		private void OnDownloadComplete(object sender, EventArgs e)
		{
			// TODO nastaveni mazat po stahnuti smaze jiz stahnuty odkaz z listu
			if (settings.getBool("osd/settings/delete_after_download", false))
			{
				downloadItems.Remove((DownloadItem) sender);
				UpdateList();
			} else
				list.RefreshObject(sender);
			UpdateDownload();
		}

		#endregion

		#region Nested type: UpdateCallback

		private delegate void UpdateCallback();

		#endregion
	}
}