﻿// $Id: Plugin.cs 19 2008-03-24 19:29:57Z arcao $
//
// Copyright (C) 2008 Martin "Arcao" Sloup <arcao@arcao.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Windows.Forms;

namespace OSD.PluginBase
{
    ///<summary>
    /// Hlavni abstraktni trida pro vytvoreni pluginu
    ///</summary>
    public abstract class Plugin
    {
        private readonly Settings settings;

        ///<summary>
        /// Konstruktor tridy s nastavenim tridy, ktera se stara o nastaveni
        ///</summary>
        ///<param name="settings">trida s nastavenim</param>
        public Plugin(Settings settings)
        {
            this.settings = settings;
        }

        ///<summary>
        /// Vraci nazev pluginu
        ///</summary>
        public abstract string Name { get; }

        ///<summary>
        /// Vraci autora pluginu
        ///</summary>
        public abstract string Author { get; }

        ///<summary>
        /// Vraci e-mail autora pluginu
        ///</summary>
        public abstract string AuthorEMail { get; }

        ///<summary>
        /// Vraci url webu, kde se nachazi plugin, pripadne jeho aktualizace
        ///</summary>
        public abstract string PluginWeb { get; }

        ///<summary>
        /// Vraci tridu s nastavenim pro plugin
        ///</summary>
        public Settings Settings
        {
            get { return settings; }
        }

        ///<summary>
        /// Obsahuje plugin dialog pro jeho podrobne nastaveni?
        ///</summary>
        public bool HasSettingDialog
        {
            get { return false; }
        }

        ///<summary>
        /// Vraci cely nazev tridy bez pocatecniho "OSD.Plugins."
        ///</summary>
        public string PluginClassName
        {
            get { return GetType().FullName.Replace("OSD.Plugins.", ""); }
        }

        ///<summary>
        /// Vraci nazev souboru (knihovny), kde se trida nachazi
        ///</summary>
        public string PluginFilename
        {
            get { return GetType().Module.Name; }
        }

        ///<summary>
        /// Zobrazi dialog s podrobnym nastavenim pluginu. Plugin, ktery chce zobrzovat dialog s podrobnym nastavenim musi tuto metodu prepsat metodou zobrazujici dialogove okno s nastavenim.
        ///</summary>
        ///<param name="owner">vlastnik okna ()</param>
        ///<returns>navratovou hodnotu okna</returns>
        ///<exception cref="NotImplementedException">Pokud neni tato metoda prepsana, vraci NotImplementedException</exception>
        public DialogResult ShowSettingDialog(IWin32Window owner)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Porovna objekt s jinym objektem. Objekty jsou stejne, pokud maji stejnou tridu a namespace.
        /// </summary>
        /// <param name="obj">objekt, se kterym se bude porovnavat</param>
        /// <returns>true, pokud je ob jekt stejne tridy a ma shodny namespace</returns>
        public override bool Equals(object obj)
        {
            if (GetType().FullName.Equals(obj.GetType().FullName))
                return true;

            return false;
        }

        ///<summary>
        ///Serves as a hash function for a particular type. <see cref="M:System.Object.GetHashCode"></see> is suitable for use in hashing algorithms and data structures like a hash table.
        ///</summary>
        ///
        ///<returns>
        ///A hash code for the current <see cref="T:System.Object"></see>.
        ///</returns>
        ///<filterpriority>2</filterpriority>
        public override int GetHashCode()
        {
            return GetType().FullName.GetHashCode();
        }

        ///<summary>
        ///Returns a <see cref="T:System.String"></see> that represents the current <see cref="T:System.Object"></see>.
        ///</summary>
        ///
        ///<returns>
        ///A <see cref="T:System.String"></see> that represents the current <see cref="T:System.Object"></see>.
        ///</returns>
        ///<filterpriority>2</filterpriority>
        public override string ToString()
        {
            return Name;
        }
    }
}