﻿// $Id: Settings.cs 19 2008-03-24 19:29:57Z arcao $
//
// Copyright (C) 2008 Martin "Arcao" Sloup <arcao@arcao.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

using System.Xml;

namespace OSD.PluginBase
{
    ///<summary>
    /// Abstraktni trida pro nastaveni
    ///</summary>
    public abstract class Settings
    {
        ///<summary>
        /// Vraci XMLDocument, kde je ulozen strom s nastavenim
        ///</summary>
        public abstract XmlDocument XmlDocument { get; }

        //public abstract void Load(string filename);
        //public abstract void Load();
        //public abstract void Save(string filename);

        ///<summary>
        /// Ulozi nastaveni programu do otevreneho Xml souboru
        ///</summary>
        public abstract void Save();

        /// <summary>
        ///  Vraci stringovou hodnotu XPath dotazu na XmlNode zadane do <code>name</code>. Pokud nastane chyba, vraci vychozi hodnotu z promene <code>default_value</code>. 
        /// </summary>
        /// <param name="name">XPath dotaz na node, kde je ulozena hodnota</param>
        /// <param name="default_value">Vychozi hodnota, pokud node neexistuje, nebo nastala-li chyba</param>
        /// <returns>Hodnota pro zadany node</returns>
        public abstract string getString(string name, string default_value);

        /// <summary>
        ///  Vraci celociselnou hodnotu XPath dotazu na XmlNode zadane do <code>name</code>. Pokud nastane chyba, vraci vychozi hodnotu z promene <code>default_value</code>. 
        /// </summary>
        /// <param name="name">XPath dotaz na node, kde je ulozena hodnota</param>
        /// <param name="default_value">Vychozi hodnota, pokud node neexistuje, nebo nastala-li chyba</param>
        /// <returns>Hodnota pro zadany node</returns>
        public abstract int getInt(string name, int default_value);

        /// <summary>
        ///  Vraci booleovskou hodnotu XPath dotazu na XmlNode zadane do <code>name</code>. Pokud nastane chyba, vraci vychozi hodnotu z promene <code>default_value</code>. 
        /// </summary>
        /// <param name="name">XPath dotaz na node, kde je ulozena hodnota</param>
        /// <param name="default_value">Vychozi hodnota, pokud node neexistuje, nebo nastala-li chyba</param>
        /// <returns>Hodnota pro zadany node</returns>
        public abstract bool getBool(string name, bool default_value);

        /// <summary>
        ///  Vraci ciselnou hodnotu XPath dotazu na XmlNode zadane do <code>name</code>. Pokud nastane chyba, vraci vychozi hodnotu z promene <code>default_value</code>. 
        /// </summary>
        /// <param name="name">XPath dotaz na node, kde je ulozena hodnota</param>
        /// <param name="default_value">Vychozi hodnota, pokud node neexistuje, nebo nastala-li chyba</param>
        /// <returns>Hodnota pro zadany node</returns>
        public abstract float getFloat(string name, float default_value);

        /// <summary>
        ///  Nastavuje stringovou hodnotu XPath dotazu na XmlNode zadane do <code>name</code>. Pokud zadana XPath cesta neexistuje, vytvori ji. 
        /// </summary>
        /// <param name="name">XPath dotaz na node, kam bude ulozena hodnota</param>
        /// <param name="value">Hodnota pro zadany node</param>
        public abstract void setString(string name, string value);

        /// <summary>
        ///  Nastavuje celociselnou hodnotu XPath dotazu na XmlNode zadane do <code>name</code>. Pokud zadana XPath cesta neexistuje, vytvori ji. 
        /// </summary>
        /// <param name="name">XPath dotaz na node, kam bude ulozena hodnota</param>
        /// <param name="value">Hodnota pro zadany node</param>
        public abstract void setInt(string name, int value);

        /// <summary>
        ///  Nastavuje booleovskou hodnotu XPath dotazu na XmlNode zadane do <code>name</code>. Pokud zadana XPath cesta neexistuje, vytvori ji. 
        /// </summary>
        /// <param name="name">XPath dotaz na node, kam bude ulozena hodnota</param>
        /// <param name="value">Hodnota pro zadany node</param>
        public abstract void setBool(string name, bool value);

        /// <summary>
        ///  Nastavuje ciselnou hodnotu XPath dotazu na XmlNode zadane do <code>name</code>. Pokud zadana XPath cesta neexistuje, vytvori ji. 
        /// </summary>
        /// <param name="name">XPath dotaz na node, kam bude ulozena hodnota</param>
        /// <param name="value">Hodnota pro zadany node</param>
        public abstract void setFloat(string name, float value);

        /// <summary>
        ///  Vrati XmlNode zadany formou jednoducheho XPath dotazu <code>path</code>. Pokud XmlNode neexistuje, vytvori ho.
        /// </summary>
        /// <param name="xmlDoc">XmlDokument, na kterem se bude pokladat dotaz</param>
        /// <param name="path">Cesta v Xml</param>
        /// <returns>Node vyhovujici prislusne ceste</returns>
        public abstract XmlNode createSelectNodes(XmlDocument xmlDoc, string path);

        /// <summary>
        ///  Vrati Node vyhovujici prislusne ceste
        /// </summary>
        /// <param name="path">Cesta v Xml stromu</param>
        /// <returns>Node vyhovujici prislusne ceste</returns>
        public abstract XmlNode getNode(string path);

        /// <summary>
        ///  Vrati NodeList vyhovujici prislusne ceste
        /// </summary>
        /// <param name="path">Cesta v Xml stromu</param>
        /// <returns>NodeList vyhovujici prislusne ceste</returns>
        public abstract XmlNodeList getNodes(string path);
    }
}